@@ -1,10 +1,16 @@
 import json
 import os
 import shutil
+import time
 import typing
 import atexit
+import re
 import distutils.dir_util as dir_util
 
+import requests
+from tqdm import tqdm
+
+
 import ExtremePluginManager.constants as constants
 import ExtremePluginManager.plugin.base_plugin as base_plugin
 import ExtremePluginManager.plugin.directory_plugin as directory_plugin
@@ -18,7 +24,7 @@ class ExtremePluginManager:
     def __init__(
             self, plugin_directory: constants.PATH_TYPEHINT, create_dir: bool = False,
             plugin_register_filename: str = "plugins.json", start_event: str = "on_start", end_event: str = "on_stop",
-            builtin_funcs: dict = None
+            builtin_funcs: dict = None, disable_events: bool = False
     ):
         if not os.path.isdir(plugin_directory):
             if create_dir:
@@ -33,6 +39,7 @@ class ExtremePluginManager:
         self._plugin_register_filename = plugin_register_filename
         self._start_event = start_event
         self._end_event = end_event
+        self._disable_events = disable_events
 
         self._builtin_funcs = {
             "print": self.stdout
@@ -46,6 +53,12 @@ class ExtremePluginManager:
 
         atexit.register(self.execute_end_event)
 
+    def disable_events(self):
+        self._disable_events = True
+
+    def enable_events(self):
+        self._disable_events = False
+
     def stdout(self, *args, sep=' ', end='\n', file=None):
         print(*args, sep=sep, end=end, file=file)
 
@@ -123,6 +136,9 @@ class ExtremePluginManager:
                 return plugin_type(self, path, self._builtin_funcs)
 
     def execute_event(self, event: str, *args, **kwargs):
+        if self._disable_events:
+            return
+
         for plugin in self._plugins:
             plugin.execute_event(event, *args, **kwargs)
 
@@ -178,6 +194,17 @@ class ExtremePluginManager:
             plugin.execute_event(self._start_event)
             self._plugins.append(plugin)
 
+    def install_plugin_url(self, url: str, enable: bool = True):
+        plugin_path = os.path.join(self._plugin_directory, str(time.time_ns()))
+
+        r = requests.get(url, stream=True)
+
+        with open(plugin_path, "wb") as handle:
+            for data in tqdm(r.iter_content()):
+                handle.write(data)
+
+        self.install_plugin(plugin_path, enable=enable, copy=False)
+
     def on_start(self):
         for plugin in self._plugins:
             plugin.execute_event(self._start_event)
